﻿using Microsoft.AspNetCore.Mvc;
using System.Net.Http;
using System.Threading.Tasks;

using IdentityModel.Client;

namespace TestAPIServer2.Controllers
{
    /// <summary>
    /// 홈 컨트롤러
    /// </summary>
    public class HomeController : Controller
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// HTTP 클라이언트 팩토리
        /// </summary>
        private readonly IHttpClientFactory factory;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - HomeController(factory)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="factory">HTTP 클라이언트 팩토리</param>
        public HomeController(IHttpClientFactory factory)
        {
            this.factory = factory;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 인덱스 페이지 처리하기 - Index()

        /// <summary>
        /// 인덱스 페이지 처리하기
        /// </summary>
        /// <returns>액션 결과 태스크</returns>
        public async Task<IActionResult> Index()
        {
            HttpClient serverClient = this.factory.CreateClient();

            DiscoveryDocumentResponse discoveryDocumentResponse = await serverClient.GetDiscoveryDocumentAsync
            (
                "https://localhost:44300/"
            );

            TokenResponse tokenResponse = await serverClient.RequestClientCredentialsTokenAsync
            (
                new ClientCredentialsTokenRequest
                {
                    Address      = discoveryDocumentResponse.TokenEndpoint,
                    ClientId     = "CLIENTID0002",
                    ClientSecret = "CLIENTSECRET0002",
                    Scope        = "API1",
                }
            );

            HttpClient apiClient = this.factory.CreateClient();

            apiClient.SetBearerToken(tokenResponse.AccessToken);

            HttpResponseMessage apiResponse = await apiClient.GetAsync("https://localhost:44310/secret/index");

            string content = await apiResponse.Content.ReadAsStringAsync();

            return Ok(new { access_token = tokenResponse.AccessToken, message = content });
        }

        #endregion
    }
}